/*
    Copyright (C) 2003 by Stephan Linz <linz@li-pro.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Bosten, MA 02111-1307 USA
*/

/* $Id: $ */

/*
   avrctrl/key.h - basic KEY pad i/o
 */

#ifndef _AVRCTRL_KEY_H_
#define _AVRCTRL_KEY_H_ 1

#if defined(HAVE_CONFIG_H) && defined(AVRCTRL_LIB_SRC)
#include <config.h>
#else
#include <avrctrl/config.h>
#endif

#ifndef __ASSEMBLER__

#include <avr/io.h>

/** \defgroup avrctrl_key Tastaturfeld Untersttzung
    \anchor a_avrctrl_key
    \code #include <avrctrl/key.h> \endcode

    Diese Headerdatei deklariert einen einfachen Low-Level Zugang zu den 5
    Tasten unterhalb des LCD. Es werden ausschlielich nur die 5 Bits
    beachtet, an denen auch Taster angeschlossen sind (Bit 3..7). Die
    restlichen Bit 0..2 des KEY Port bleiben unberhrt.

    Vor der Benutzung der KEY Funktionen, mu dieser Teil der Bibliothek mit
    key_init() initialisiert werden. Die Bibliothek stellt fr die Ermittlung
    gedrckter Tasten die Funktion key_scancode() bereit. Jeder Taste ist im
    Scancode ein Bit zugeordnet. Fr die einfache Auswertung sind diese Codes
    Bestandteil dieser Deklaration.

    \todo Parametrisierung ber mehrere Ports verstreuter Bits. */

/** \name Low-Level Zugriff */

/*@{*/

/** \def key_init()
    \ingroup avrctrl_key
    Initialisiert den KEY Port zur Biteingabe (alle 5 Bits). Dabei bleiben
    die unteren Bits unbehandelt.
    
    \return Das Makro key_init() besitzt keinen Rckgabewert. */
#define key_init()						\
	AVRCTRL_LIB_KEY_PORT_DDR =				\
		(AVRCTRL_LIB_KEY_PORT_DDR & ~KEY_SCAN_ALL) | 0x00;

/** \def key_scancode()
    \ingroup avrctrl_key
    Liet den aktuellen Zustand der Tasten ein und liefert einen
    entsprechenden Scancode.
    
    \return Das Makro key_scancode() gibt den aktuellen Scancode als
    \c unsigned \c char zurck. */
#define key_scancode()					\
	(unsigned char)(AVRCTRL_LIB_KEY_PORT_IN & KEY_SCAN_ALL)

/*@}*/

/** \name Scancodes */

/*@{*/

/** \def KEY_SCAN_T1
    \ingroup avrctrl_key
    Bitwert fr Taste T1, erste von links. */
#define KEY_SCAN_T1		_BV(7)

/** \def KEY_SCAN_T2
    \ingroup avrctrl_key
    Bitwert fr Taste T2, zweite von links. */
#define KEY_SCAN_T2		_BV(6)

/** \def KEY_SCAN_T3
    \ingroup avrctrl_key
    Bitwert fr Taste T3, dritte von links. */
#define KEY_SCAN_T3		_BV(5)

/** \def KEY_SCAN_T4
    \ingroup avrctrl_key
    Bitwert fr Taste T4, vierte von links. */
#define KEY_SCAN_T4		_BV(4)

/** \def KEY_SCAN_T5
    \ingroup avrctrl_key
    Bitwert fr Taste T5, fnfte von links. */
#define KEY_SCAN_T5		_BV(3)

/** \def KEY_SCAN_ALL
    \ingroup avrctrl_key
    Bitwert fr alle Tasten T1 bis T5 (Tastenmaske). */
#define KEY_SCAN_ALL		(	KEY_SCAN_T1	\
				|	KEY_SCAN_T2	\
				|	KEY_SCAN_T3	\
				|	KEY_SCAN_T4	\
				|	KEY_SCAN_T5	)

/*@}*/

#endif /* __ASSEMBLER__ */

#endif /* _AVRCTRL_KEY_H_ */

/*************************************************************************
 *
 *      KEY access control
 *
 *************************************************************************
 *
 * key_scancode:
 *
 *      Bit:   7 6 5 4 3 2 1 0
 *             | | | | | | | \___ not used
 *             | | | | | | \_____ not used
 *             | | | | | \_______ not used
 *             | | | | \_________ T5
 *             | | | \___________ T4
 *             | | \_____________ T3
 *             | \_______________ T2
 *             \_________________ T1
 *
 *
 * AVR-Ctrl key numbers:
 *              _______________________________
 *             |                               |
 *             |      LCD ...                  |
 *             |_______________________________|
 *               _____ _____ _____ _____ _____
 *              |  T1 |  T2 |  T3 |  T4 |  T5 |
 *              |_____|_____|_____|_____|_____|
 *              |\___/|\___/|\___/|\___/|\___/|
 *              ||___|||___|||___|||___|||___||
 *              |/___\|/___\|/___\|/___\|/___\|
 *
 ************************************************************************/

