/*
    Copyright (C) 2003 by Stephan Linz <linz@li-pro.net>

    Some code comes from Brajer Vlado <vlado.brajer@kks.s-net.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Bosten, MA 02111-1307 USA
*/

/* $Id: $ */

/*
   avrctrl/led.h - basic LCD ascii i/o
 */

#ifndef _AVRCTRL_LCD_H_
#define _AVRCTRL_LCD_H_ 1

#if defined(HAVE_CONFIG_H) && defined(AVRCTRL_LIB_SRC)
#include <config.h>
#else
#include <avrctrl/config.h>
#endif

#ifndef __ASSEMBLER__

#include <avr/io.h>
#include <avrctrl/delay.h>

/** \defgroup avrctrl_lcd LCD Untersttzung
    \anchor a_avrctrl_lcd
    \code #include <avrctrl/led.h> \endcode

    Diese Headerdatei deklariert zunchst einen Low-Level Zugang fr das LCD.
    Es werden dabei 1 bis 4 Zeilen Displays mit bis zu 64 Zeichen je Zeile
    untersttzt (HITACHI LCD). Es wird immer von einer Nibble Kommunikation
    mit dem LCD ausgegangen. Die Steuersignale RS, RD und EN sind am selben
    LCD Port frei definierbar. Das eine unbenutzte Bit ist wie die
    Steuersignale zum Zeitpunkt der bersetzung parametrisierbar und wird
    zur Laufzeit bercksichtigt.

    Vor der Benutzung der LCD Funktionen, mu dieser Teil der Bibliothek mit
    lcd_init() initialisiert werden. Hierbei wird der Typ des LCD
    verschlsselt bergeben. Dieser Schlssel, lcd_matrix, hat folgenden
    Aufbau:
	- Bit 5..0:
		- Anzahl der Zeichen je Zeile
		- Werte zwischen 0 und 63, z.B.:
			- 0x10: 16 Zeichen
	- Bit 7..6:
		- Anzahl Zeilen
		- Werte zwischen 0 und 3, z.B.:
			- 0: wie 3 (Kompatibilitt)
			- 1: 1 Zeile
			- 2: 2 Zeilen
			- 3: 4 Zeilen

    \todo Beseitigung der unklaren Nummerierung fr die Anzahl Zeichen je
    Zeile in lcd_matrix, oder anders: Was ist bei Anzahl gleich Null(0) ? */

/** \name Low-Level Zugriff / Brajer Vlado Konformitt */

/*@{*/

#ifdef __cplusplus
extern "C" {
#endif

/** \fn void lcd_init(unsigned char lcd_matrix)
    \ingroup avrctrl_lcd
    \brief ...

    ...
    ...
    ...

    ...
    ...
    ...

    \param lcd_matrix definiert den Typ des LCD.
    Siehe \ref a_avrctrl_lcd "Gruppenbeschreibung".

    \return Die Funktion lcd_init() besitzt keinen Rckgabewert. */
extern void lcd_init(unsigned char lcd_matrix);

/** \fn void lcd_cls(void)
    \ingroup avrctrl_lcd
    \brief ...

    ...
    ...
    ...
    
    \return Die Funktion lcd_cls() besitzt keinen Rckgabewert. */
extern void lcd_cls(void);

/** \fn void lcd_home(void)
    \ingroup avrctrl_lcd
    \brief ...

    ...
    ...
    ...
    
    \return Die Funktion lcd_home() besitzt keinen Rckgabewert. */
extern void lcd_home(void);

/** \fn void lcd_goto(unsigned char addr)
    \ingroup avrctrl_lcd
    \brief ...

    ...
    ...
    ...

    \param addr gibt die Adresse im Datenspeicher des LCD an.
    
    \return Die Funktion lcd_goto() besitzt keinen Rckgabewert. */
extern void lcd_goto(unsigned char addr);

/** \fn void lcd_ctrl(unsigned char control)
    \ingroup avrctrl_lcd
    \brief ...

    ...
    ...
    ...

    Bit 0 : cursor blink on(1) / off(0)
    Bit 1 : cursor on(1) / off(0)
    Bit 2 : display on(1) / off(0)
    
    \param control ist die Maske mit den enthaltenen Anweisungen.

    \return Die Funktion lcd_ctrl() besitzt keinen Rckgabewert. */
extern void lcd_ctrl(unsigned char control);

/** \fn void lcd_putch(unsigned char data)
    \ingroup avrctrl_lcd
    \brief ...

    ...
    ...
    ...

    \param data ist das auszugebende Zeichen.
    
    \return Die Funktion lcd_putch() besitzt keinen Rckgabewert. */
extern void lcd_putch(unsigned char data);

/** \fn void lcd_print10(unsigned long x)
    \ingroup avrctrl_lcd
    \brief ...

    ...
    ...
    ...

    \param x ist die auszugebende Zahl.
    
    \return Die Funktion lcd_print10() besitzt keinen Rckgabewert. */
extern void lcd_print10(unsigned long x);

/** \fn void lcd_print2(unsigned char x)
    \ingroup avrctrl_lcd
    \brief ...

    ...
    ...
    ...

    \param x ist die auszugebende Zahl.
    
    \return Die Funktion lcd_print2() besitzt keinen Rckgabewert. */
extern void lcd_print2(unsigned char x);

/** \fn void lcd_print3(unsigned int x)
    \ingroup avrctrl_lcd
    \brief ...

    ...
    ...
    ...

    \param x ist die auszugebende Zahl.
    
    \return Die Funktion lcd_print3() besitzt keinen Rckgabewert. */
extern void lcd_print3(unsigned int x);

/** \fn void lcd_print5(unsigned int x)
    \ingroup avrctrl_lcd
    \brief ...

    ...
    ...
    ...

    \param x ist die auszugebende Zahl.
    
    \return Die Funktion lcd_print5() besitzt keinen Rckgabewert. */
extern void lcd_print5(unsigned int x);

/** \fn void lcd_printbin(unsigned char x)
    \ingroup avrctrl_lcd
    \brief ...

    ...
    ...
    ...

    \param x ist die auszugebende Zahl.
    
    \return Die Funktion lcd_printbin() besitzt keinen Rckgabewert. */
extern void lcd_printbin(unsigned char x);

/** \fn void lcd_printf(const char * format, ...)
    \ingroup avrctrl_lcd
    \brief ...

    ...
    ...
    ...

    \param format ist der Formatstring.
    
    \return Die Funktion lcd_printf() besitzt keinen Rckgabewert. */
extern void lcd_printf(const char * format, ...);

/** \fn void lcd_printhex(unsigned char x)
    \ingroup avrctrl_lcd
    \brief ...

    ...
    ...
    ...

    \param x ist die auszugebende Zahl.
    
    \return Die Funktion lcd_printhex() besitzt keinen Rckgabewert. */
extern void lcd_printhex(unsigned char x);

/** \fn void lcd_putstr(unsigned char * data)
    \ingroup avrctrl_lcd
    \brief ...

    ...
    ...
    ...

    \param data zeigt auf die auszugebende Zeichenkette.
    
    \return Die Funktion lcd_putstr() besitzt keinen Rckgabewert. */
extern void lcd_putstr(unsigned char * data);

#ifdef __cplusplus
}
#endif

/** \def lcd_control(D,C,B)
    \ingroup avrctrl_lcd
    ...

    \param D gibt mit ungleich Null an, ob das Display aktiv ist.
    \param C gibt mit ungleich Null an, ob der Cursor aktiv ist.
    \param B gibt mit ungleich Null an, ob der Cursor blinkt.

    \return Das Makro lcd_control() besitzt keinen Rckgabewert. */
#define lcd_control(D,C,B)	lcd_ctrl((unsigned char)(	\
						( (D) << 2 )	\
					|	( (C) << 1 )	\
					|	  (B)		))

/*@}*/

/** \name CodeVision Konformitt */

/*@{*/

#ifdef __cplusplus
extern "C" {
#endif

/** \fn void lcd_gotoxy(unsigned char x, unsigned char y)
    \ingroup avrctrl_lcd
    \brief ...

    ...
    ...
    ...

    \param x gibt die X-Koordinate im Datenspeicher des LCD an.
    \param y gibt die Y-Koordinate im Datenspeicher des LCD an.
    
    \return Die Funktion lcd_gotoxy() besitzt keinen Rckgabewert. */
extern void lcd_gotoxy(unsigned char x, unsigned char y);

/** \fn void lcd_write_byte(unsigned char addr, unsigned char data)
    \ingroup avrctrl_lcd
    \brief ...

    ...
    ...
    ...

    \param addr gibt die Adresse im Datenspeicher des LCD an.
    \param data ist das auszugebende Zeichen.
    
    \return Die Funktion lcd_write_byte() besitzt keinen Rckgabewert. */
extern void lcd_write_byte(unsigned char addr, unsigned char data);

/** \fn void _lcd_write_data(unsigned char data)
    \ingroup avrctrl_lcd
    \brief ...

    ...
    ...
    ...

    \param data ist das auszugebende Zeichen.
    
    \return Die Funktion _lcd_write_data() besitzt keinen Rckgabewert. */
extern void _lcd_write_data(unsigned char data);

#ifdef __cplusplus
}
#endif

/** \def lcd_clear()
    \ingroup avrctrl_lcd
    ...
    
    \return Das Makro lcd_clear() besitzt keinen Rckgabewert. */
#define lcd_clear()		lcd_cls()

/** \def lcd_putchar()
    \ingroup avrctrl_lcd
    ...
    
    \return Das Makro lcd_putchar() besitzt keinen Rckgabewert. */
#define lcd_putchar(C)		lcd_putch((unsigned char)(C))

/** \def lcd_puts()
    \ingroup avrctrl_lcd
    ...
    
    \return Das Makro lcd_puts() besitzt keinen Rckgabewert. */
#define lcd_puts(S)		lcd_putstr((unsigned char *)(S))

/** \def _lcd_ready()
    \ingroup avrctrl_lcd
    Wartet, bis das LCD zur Kommunikation bereit ist.
    
    \return Das Makro _lcd_ready() besitzt keinen Rckgabewert. */
#ifdef AVRCTRL_LIB_LCD_COM_POLLING
#define _lcd_ready()		_lcd_check_BF()
#else
#warning "AVRCTRL_LIB_LCD_COM_POLLING not defined -- use simulated _lcd_ready()"
#define _lcd_ready()		delay_ms(2)
#endif

/*@}*/

#endif /* __ASSEMBLER__ */

#endif /* _AVRCTRL_LCD_H_ */
